@@ -1,20 +1,22 @@
-/*
-    Copyright (C) 2008 Fons Adriaensen <fons@kokkinizita.net>
-    
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+// ----------------------------------------------------------------------
+//
+//  Copyright (C) 2008-2012 Fons Adriaensen <fons@linuxaudio.org>
+//    
+//  This program is free software; you can redistribute it and/or modify
+//  it under the terms of the GNU General Public License as published by
+//  the Free Software Foundation; either version 2 of the License, or
+//  (at your option) any later version.
+//
+//  This program is distributed in the hope that it will be useful,
+//  but WITHOUT ANY WARRANTY; without even the implied warranty of
+//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//  GNU General Public License for more details.
+//
+//  You should have received a copy of the GNU General Public License
+//  along with this program; if not, write to the Free Software
+//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+//
+// ----------------------------------------------------------------------
 
 
 #include <stdlib.h>
@@ -27,7 +29,6 @@
 #include "png2img.h"
 
 
-
 Mainwin::Mainwin (X_rootwin *parent, X_resman *xres, Jclient *jclient, JMconf *config) :
     A_thread ("Main"),
     X_window (parent, 100, 100, 100, 100, XftColors [C_MAIN_BG]->pixel),
@@ -38,9 +39,10 @@ Mainwin::Mainwin (X_rootwin *parent, X_resman *xres, Jclient *jclient, JMconf *c
     _config (config),
     _dx (0),
     _dy (0),
-    _dt (50000)
+    _dt (50000),
+    _dg (1)
 {
-    int       xp, yp, xs, ys;
+    int       xp, yp, xs, ys, xx, yy;
     int       i, j, x, y;
     char      s [1024];
     X_hints   H;
@@ -49,68 +51,33 @@ Mainwin::Mainwin (X_rootwin *parent, X_resman *xres, Jclient *jclient, JMconf *c
     XSetWMProtocols (dpy (), win (), &_atom, 1);
     _atom = XInternAtom (dpy (), "WM_PROTOCOLS", True);
 
-    switch (config->_type)
-    {
-    case Meterwin::MVUM:
-    case Meterwin::SVUM:
-	makepixm (SHARED "/jmeters/vu-meter1.png", XftColors [C_MAIN_BG]);
-	break;
-
-    case Meterwin::MPPM:
-    case Meterwin::SPPM:
-	makepixm (SHARED "/jmeters/ppm-meter1.png", XftColors [C_MAIN_BG]);
-	break;
-
-    case Meterwin::MPPM2:
-    case Meterwin::SPPM2:
-	makepixm (SHARED "/jmeters/ppm-meter-orig.png", XftColors [C_MAIN_BG]);
-	break;
-    }
-    if (_dx == 0)
-    {
-	_stop = true;
-	return;
-    }
-
-    j = atoi (xres->get (".rate", "20"));
+    j = atoi (xres->get (".update", "20"));
     if (j < 10) j = 10;
     if (j > 50) j = 50;
     _dt = 1000000 / j;
-
-    _nc = _nm = _config->_numb;
-    if (config->_type & 1) _nm = (_nc + 1) / 2;
+    _dg = powf (10.0f, 0.05f * _config->_gain);
+    _nc = config->_ninp;
+    _nm = _nc / _config->_nchan;;
     j = y = 0;
-    x = 8;
+    x = 0;
     for (i = 0; i < _nm; i++) 
     {
 	if (j == config->_ncol)
 	{
 	    j = 0;
-            x = 8;
+            x = 0;
 	    y += _dy;
 	}
-
-	_meters [i] = new Meterwin (this, x, y, config->_type, XftColors [C_MAIN_BG], _pixm);
+	_meters [i] = new Meterwin (this, x, y, XftColors [C_MAIN_BG],
+                                    _config->_mtype, _config->_frame, _config->_nchan,
+                                    SHARED, _config->_label [i]);
+	if (i == 0)
+	{
+	    _dx = _meters [i]->xs ();
+	    _dy = _meters [i]->ys ();
+	}
 	x += _dx;
 	j += 1;
-
-	switch (config->_type)
-	{
-	case Meterwin::MVUM:
-	    _meters [i]->set_col (XftColors [C_VUM1], 0);
-	    break;
-	case Meterwin::SVUM:
-	    _meters [i]->set_col (XftColors [C_VUML], XftColors [C_VUMR]);
-	    break;
-	case Meterwin::MPPM:
-	case Meterwin::MPPM2:
-	    _meters [i]->set_col (XftColors [C_PPM1], 0);
-	    break;
-	case Meterwin::SPPM:
-	case Meterwin::SPPM2:
-	    _meters [i]->set_col (XftColors [C_PPML], XftColors [C_PPMR]);
-	    break;
-	}
     }	
 
     sprintf (s, "%s    %s-%s", jclient->jname (), PROGRAM, VERSION);
@@ -118,15 +85,15 @@ Mainwin::Mainwin (X_rootwin *parent, X_resman *xres, Jclient *jclient, JMconf *c
     j = (_nm < config->_ncol) ? _nm : config->_ncol;
     xp = 100;
     yp = 100;
-    xs = j * _dx + 16;
+    xs = j * _dx;
     ys = y + _dy;
-    H.position (0, 0);
+    H.position (100, 100);
     H.minsize (100, 100);
     H.maxsize (xs, ys);
     H.rname (xres->rname ());
     H.rclas (xres->rclas ());
     x_apply (&H); 
-    xres->geometry (".geometry", disp ()->xsize (), disp ()->ysize (), 1, xp, yp, xs, ys);
+    xres->geometry (".geometry", disp ()->xsize (), disp ()->ysize (), 1, xp, yp, xx, yy);
     x_moveresize (xp, yp, xs, ys);
 
     x_map (); 
@@ -188,18 +155,18 @@ void Mainwin::clmesg (XClientMessageEvent *E)
 
 void Mainwin::handle_time (void)
 {
-    int i, j;
+    int i, j, n;
     float v1, v2;
 
+    n = _config->_nchan;
     for (i = j = 0; i < _nm; i++)
     {
-	v1 = _config->_mdata [j++]._level; 
-	if (_nc > _nm) v2 = _config->_mdata [j++]._level;
+	if (n >= 1) v1 = _jclient->read (j++);
+	else v1 = 0;
+	if (n >= 2) v2 = _jclient->read (j++);  
 	else v2 = 0;
-	_meters [i]->set_val (v1, v2);
-	_meters [i]->update ();
+	_meters [i]->set_val (_dg * v1, _dg * v2);
     }
-    _config->_cnt1 = _config->_cnt2 + 1;
 
     XFlush (dpy ());
     inc_time (_dt);
@@ -227,22 +194,6 @@ void Mainwin::redraw (void)
 }
 
 
-void Mainwin::makepixm (const char *file, XftColor *bgnd)
-{
-    XImage *image;
-
-    image = png2img (file, disp (), bgnd);
-    if (! image) return;
-    _dx = image->width - 4;
-    _dy = image->height - 4;
-    _pixm = XCreatePixmap (dpy (), win (), _dx, _dy, disp ()->depth ());
-    if (_pixm) XPutImage (dpy (), _pixm, dgc (), image, 2, 2, 0, 0, _dx, _dy); 
-    delete[] image->data;
-    image->data = 0;
-    XDestroyImage (image);
-}
-
-
 void Mainwin::addtext (X_window *W, X_textln_style *T, int xp, int yp, int xs, int ys, const char *text, int align)
 {
     (new X_textln (W, T, xp, yp, xs, ys, text, align))->x_map ();
